# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO
import satnogsdecoders.process


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Chomptt(KaitaiStruct):
    """:field dest_callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.payload.pid
    :field identifier: ax25_frame.payload.ax25_info.identifier
    :field msg_num: ax25_frame.payload.ax25_info.chomptb85.msg_num
    :field time: ax25_frame.payload.ax25_info.chomptb85.time
    :field mode: ax25_frame.payload.ax25_info.chomptb85.mode
    :field idle_timer: ax25_frame.payload.ax25_info.chomptb85.idle_timer
    :field vbat: ax25_frame.payload.ax25_info.chomptb85.vbat
    :field spc_xp: ax25_frame.payload.ax25_info.chomptb85.spc_xp
    :field spc_xn: ax25_frame.payload.ax25_info.chomptb85.spc_xn
    :field spc_yp: ax25_frame.payload.ax25_info.chomptb85.spc_yp
    :field spc_zp: ax25_frame.payload.ax25_info.chomptb85.spc_zp
    :field spc_zn: ax25_frame.payload.ax25_info.chomptb85.spc_zn
    :field temp_b1: ax25_frame.payload.ax25_info.chomptb85.temp_b1
    :field temp_b3: ax25_frame.payload.ax25_info.chomptb85.temp_b3
    :field temp_b4: ax25_frame.payload.ax25_info.chomptb85.temp_b4
    :field temp_b5: ax25_frame.payload.ax25_info.chomptb85.temp_b5
    :field temp_b6: ax25_frame.payload.ax25_info.chomptb85.temp_b6
    :field temp_b9: ax25_frame.payload.ax25_info.chomptb85.temp_b9
    :field temp_xn: ax25_frame.payload.ax25_info.chomptb85.temp_xn
    :field temp_xp: ax25_frame.payload.ax25_info.chomptb85.temp_xp
    :field temp_yp: ax25_frame.payload.ax25_info.chomptb85.temp_yp
    :field temp_zn: ax25_frame.payload.ax25_info.chomptb85.temp_zn
    :field temp_zp: ax25_frame.payload.ax25_info.chomptb85.temp_zp
    :field v_bat: ax25_frame.payload.ax25_info.chomptb85.v_bat
    :field i_dog: ax25_frame.payload.ax25_info.chomptb85.i_dog
    :field i_sat: ax25_frame.payload.ax25_info.chomptb85.i_sat
    :field i_temp: ax25_frame.payload.ax25_info.chomptb85.i_temp
    :field i_gps_arduino: ax25_frame.payload.ax25_info.chomptb85.i_gps_arduino
    :field i_gps_novatel: ax25_frame.payload.ax25_info.chomptb85.i_gps_novatel
    :field i_sten: ax25_frame.payload.ax25_info.chomptb85.i_sten
    :field i_acs: ax25_frame.payload.ax25_info.chomptb85.i_acs
    :field i_rout: ax25_frame.payload.ax25_info.chomptb85.i_rout
    :field i_lit: ax25_frame.payload.ax25_info.chomptb85.i_lit
    :field mag_x: ax25_frame.payload.ax25_info.chomptb85.mag_x
    :field mag_y: ax25_frame.payload.ax25_info.chomptb85.mag_y
    :field mag_z: ax25_frame.payload.ax25_info.chomptb85.mag_z
    :field identifier: ax25_frame.payload.ax25_info.identifier
    :field flags: ax25_frame.payload.ax25_info.optib85.flags
    :field sup_t1: ax25_frame.payload.ax25_info.optib85.sup_t1
    :field sup_t2: ax25_frame.payload.ax25_info.optib85.sup_t2
    :field sup_vbat: ax25_frame.payload.ax25_info.optib85.sup_vbat
    :field ch1_flags: ax25_frame.payload.ax25_info.optib85.ch1_flags
    :field ch1_temp: ax25_frame.payload.ax25_info.optib85.ch1_temp
    :field csac1_temp: ax25_frame.payload.ax25_info.optib85.csac1_temp
    :field ch1_current: ax25_frame.payload.ax25_info.optib85.ch1_current
    :field ch2_flags: ax25_frame.payload.ax25_info.optib85.ch2_flags
    :field ch2_temp: ax25_frame.payload.ax25_info.optib85.ch2_temp
    :field csac2_temp: ax25_frame.payload.ax25_info.optib85.csac2_temp
    :field ch2_current: ax25_frame.payload.ax25_info.optib85.ch2_current
    :field count: ax25_frame.payload.ax25_info.optib85.count
    :field overflow: ax25_frame.payload.ax25_info.optib85.overflow
    
    .. seealso::
       Source - https://pssl.mae.ufl.edu/#chomptt/page/beacondecode
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Chomptt.Ax25Frame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Chomptt.Ax25Header(self._io, self, self._root)
            _on = (self.ax25_header.ctl & 19)
            if _on == 0:
                self.payload = Chomptt.IFrame(self._io, self, self._root)
            elif _on == 3:
                self.payload = Chomptt.UiFrame(self._io, self, self._root)
            elif _on == 19:
                self.payload = Chomptt.UiFrame(self._io, self, self._root)
            elif _on == 16:
                self.payload = Chomptt.IFrame(self._io, self, self._root)
            elif _on == 18:
                self.payload = Chomptt.IFrame(self._io, self, self._root)
            elif _on == 2:
                self.payload = Chomptt.IFrame(self._io, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Chomptt.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Chomptt.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Chomptt.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Chomptt.SsidMask(self._io, self, self._root)
            if (self.src_ssid_raw.ssid_mask & 1) == 0:
                self.repeater = Chomptt.Repeater(self._io, self, self._root)

            self.ctl = self._io.read_u1()


    class UiFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            _on = self._root.framelength
            if _on == 69:
                self._raw_ax25_info = self._io.read_bytes_full()
                _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
                self.ax25_info = Chomptt.Optib85T(_io__raw_ax25_info, self, self._root)
            elif _on == 128:
                self._raw_ax25_info = self._io.read_bytes_full()
                _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
                self.ax25_info = Chomptt.Chomptb85T(_io__raw_ax25_info, self, self._root)
            else:
                self.ax25_info = self._io.read_bytes_full()


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


    class IFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self.ax25_info = self._io.read_bytes_full()


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return getattr(self, '_m_ssid', None)


    class Repeaters(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rpt_callsign_raw = Chomptt.CallsignRaw(self._io, self, self._root)
            self.rpt_ssid_raw = Chomptt.SsidMask(self._io, self, self._root)


    class Repeater(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rpt_instance = []
            i = 0
            while True:
                _ = Chomptt.Repeaters(self._io, self, self._root)
                self.rpt_instance.append(_)
                if (_.rpt_ssid_raw.ssid_mask & 1) == 1:
                    break
                i += 1


    class Optib85T(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.identifier = (self._io.read_bytes(4)).decode(u"ASCII")
            self.sod = self._io.read_bytes(2)
            self._raw__raw_optib85 = self._io.read_bytes(38)
            _process = satnogsdecoders.process.B85decode()
            self._raw_optib85 = _process.decode(self._raw__raw_optib85)
            _io__raw_optib85 = KaitaiStream(BytesIO(self._raw_optib85))
            self.optib85 = Chomptt.OptiT(_io__raw_optib85, self, self._root)
            self.eod = self._io.read_bytes(2)


    class OptiT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.flags = self._io.read_u2le()
            self.sup_t1 = self._io.read_s2le()
            self.sup_t2 = self._io.read_s2le()
            self.sup_vbat = self._io.read_s2le()
            self.ch1_flags = self._io.read_u2le()
            self.ch1_temp = self._io.read_s2le()
            self.csac1_temp = self._io.read_s2le()
            self.ch1_current = self._io.read_s2le()
            self.ch2_flags = self._io.read_u2le()
            self.ch2_temp = self._io.read_s2le()
            self.csac2_temp = self._io.read_s2le()
            self.ch2_current = self._io.read_s2le()
            self.count = self._io.read_u2le()
            self.overflow = self._io.read_u4le()


    class ChomptT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.msg_num = self._io.read_u2le()
            self.time = self._io.read_u4le()
            self.mode = self._io.read_u1()
            self.idle_timer = self._io.read_u2le()
            self.vbat = self._io.read_u2le()
            self.spc_xp = self._io.read_s2le()
            self.spc_xn = self._io.read_s2le()
            self.spc_yp = self._io.read_s2le()
            self.spc_zp = self._io.read_s2le()
            self.spc_zn = self._io.read_s2le()
            self.temp_b1 = self._io.read_u2le()
            self.temp_b3 = self._io.read_u2le()
            self.temp_b4 = self._io.read_u2le()
            self.temp_b5 = self._io.read_u2le()
            self.temp_b6 = self._io.read_u2le()
            self.temp_b9 = self._io.read_u2le()
            self.temp_xn = self._io.read_s2le()
            self.temp_xp = self._io.read_s2le()
            self.temp_yp = self._io.read_s2le()
            self.temp_zn = self._io.read_s2le()
            self.temp_zp = self._io.read_s2le()
            self.v_bat = self._io.read_s2le()
            self.i_dog = self._io.read_s2le()
            self.i_sat = self._io.read_s2le()
            self.i_temp = self._io.read_s2le()
            self.i_gps_arduino = self._io.read_s2le()
            self.i_gps_novatel = self._io.read_s2le()
            self.i_sten = self._io.read_s2le()
            self.i_acs = self._io.read_s2le()
            self.i_rout = self._io.read_s2le()
            self.i_lit = self._io.read_s2le()
            self.mag_x = self._io.read_f4le()
            self.mag_y = self._io.read_f4le()
            self.mag_z = self._io.read_f4le()


    class Chomptb85T(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.identifier = (self._io.read_bytes_term(92, False, True, True)).decode(u"ASCII")
            self.sod = self._io.read_bytes(2)
            self._raw__raw_chomptb85 = self._io.read_bytes(94)
            _process = satnogsdecoders.process.B85decode()
            self._raw_chomptb85 = _process.decode(self._raw__raw_chomptb85)
            _io__raw_chomptb85 = KaitaiStream(BytesIO(self._raw_chomptb85))
            self.chomptb85 = Chomptt.ChomptT(_io__raw_chomptb85, self, self._root)
            self.eod = self._io.read_bytes(2)


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Chomptt.Callsign(_io__raw_callsign_ror, self, self._root)


    @property
    def rpt1_callsign(self):
        if hasattr(self, '_m_rpt1_callsign'):
            return self._m_rpt1_callsign

        self._m_rpt1_callsign = self.ax25_frame.ax25_header.repeater.rpt_instance[0].rpt_callsign_raw.callsign_ror.callsign
        return getattr(self, '_m_rpt1_callsign', None)

    @property
    def framelength(self):
        if hasattr(self, '_m_framelength'):
            return self._m_framelength

        self._m_framelength = self._io.size()
        return getattr(self, '_m_framelength', None)





"""
SatNOGS Decoder subpackage initialization
"""
from __future__ import absolute_import, division, print_function

import enum
import functools
import re

from .aausat4 import Aausat4
from .acrux1 import Acrux1
from .aepex import Aepex
from .alsat1n import Alsat1n
from .amicalsat import Amicalsat
from .armadillo import Armadillo
from .ascii85test import Ascii85test
from .astrocast import Astrocast
from .asuphoenix import Asuphoenix
from .ax25frames import Ax25frames
from .ax25monitor import Ax25monitor
from .azaadisat2 import Azaadisat2
from .bdsat import Bdsat
from .bdsat2 import Bdsat2
from .beesat import Beesat
from .beesat2 import Beesat2
from .bisonsat import Bisonsat
from .bobcat1 import Bobcat1
from .bugsat1 import Bugsat1
from .cape1 import Cape1
from .cas4 import Cas4
from .cas5a import Cas5a
from .cas9 import Cas9
from .catsat import Catsat
from .celesta import Celesta
from .chomptt import Chomptt
from .cirbe import Cirbe
from .co65 import Co65
from .connectat11 import Connectat11
from .cosmogirlsat import Cosmogirlsat
from .crocube import Crocube
from .csim import Csim
from .ctim import Ctim
from .cubebel1 import Cubebel1
from .cubebel2 import Cubebel2
from .cubesatsim import Cubesatsim
from .cute import Cute
from .delfin3xt import Delfin3xt
from .delfipq import Delfipq
from .dhabisat import Dhabisat
from .diy1 import Diy1
from .dora import Dora
from .duchifat3 import Duchifat3
from .eirsat1 import Eirsat1
from .elfin import Elfin
from .enso import Enso
from .entrysat import Entrysat
from .equisat import Equisat
from .eshail2 import Eshail2
from .estcube2 import Estcube2
from .foresail1 import Foresail1
from .fox import Fox
from .gaspacs import Gaspacs
from .geoscan import Geoscan
from .geoscanedelveis import Geoscanedelveis
from .grbalpha import Grbalpha
from .grbbeta import Grbbeta
from .greencube import Greencube
from .grizu263a import Grizu263a
from .gt1 import Gt1
from .hadesd import Hadesd
from .inspiresat1 import Inspiresat1
from .io86 import Io86
from .irazu import Irazu
from .irvine import Irvine
from .iss import Iss
from .kafasat1 import Kafasat1
from .kashiwa import Kashiwa
from .kosen1 import Kosen1
from .ksu import Ksu
from .lasarsat import Lasarsat
from .ledsat import Ledsat
from .lightsail2 import Lightsail2
from .lucky7 import Lucky7
from .meznsat import Meznsat
from .minxss import Minxss
from .mirsat1 import Mirsat1
from .mitee1 import Mitee1
from .mxl import Mxl
from .mysat import Mysat
from .netsat import Netsat
from .neudose import Neudose
from .neutron1 import Neutron1
from .no44 import No44
from .nutsat1 import Nutsat1
from .opssat1 import Opssat1
from .oresat0 import Oresat0
from .oresat0_5 import Oresat05
from .origamisat1 import Origamisat1
from .painani import Painani
from .pegasus import Pegasus
from .picsat import Picsat
from .planetum1 import Planetum1
from .polyitan1 import Polyitan1
from .pwsat2 import Pwsat2
from .qarman import Qarman
from .qbee import Qbee
from .qube import Qube
from .qubik import Qubik
from .quetzal1 import Quetzal1
from .ramsat import Ramsat
from .randev import Randev
from .rhoksat import Rhoksat
from .roseycubesat1 import Roseycubesat1
from .sakura import Sakura
from .salsat import Salsat
from .sanosat1 import Sanosat1
from .selfiesat import Selfiesat
from .sharjahsat1 import Sharjahsat1
from .siriussat import Siriussat
from .skcube import Skcube
from .snet import Snet
from .spoc import Spoc
from .sputnixusp import Sputnixusp
from .sr0 import Sr0
from .strand import Strand
from .stratosattk1 import Stratosattk1
from .suchai2 import Suchai2
from .targit import Targit
from .us6 import Us6
from .uwe4 import Uwe4
from .veronika import Veronika
from .vzlusat2 import Vzlusat2
from .yomogi import Yomogi

__all__ = [
    'Aausat4',
    'Acrux1',
    'Aepex',
    'Alsat1n',
    'Amicalsat',
    'Armadillo',
    'Ascii85test',
    'Astrocast',
    'Asuphoenix',
    'Ax25frames',
    'Ax25monitor',
    'Azaadisat2',
    'Bdsat',
    'Bdsat2',
    'Beesat',
    'Beesat2',
    'Bisonsat',
    'Bobcat1',
    'Bugsat1',
    'Cape1',
    'Cas4',
    'Cas5a',
    'Cas9',
    'Catsat',
    'Celesta',
    'Chomptt',
    'Co65',
    'Connectat11',
    'Cosmogirlsat',
    'Crocube',
    'Cubebel1',
    'Cubebel2',
    'Cubesatsim',
    'Cute',
    'Csim',
    'Ctim',
    'Cirbe',
    'Delfin3xt',
    'Delfipq',
    'Dhabisat',
    'Diy1',
    'Dora',
    'Duchifat3',
    'Eirsat1',
    'Elfin',
    'Enso',
    'Entrysat',
    'Eshail2',
    'Equisat',
    'Estcube2',
    'Foresail1',
    'Fox',
    'Gaspacs',
    'Geoscan',
    'Geoscanedelveis',
    'Grbalpha',
    'Grbbeta',
    'Greencube',
    'Grizu263a',
    'Gt1',
    'Hadesd',
    'Inspiresat1',
    'Io86',
    'Irazu',
    'Irvine',
    'Iss',
    'Kafasat1',
    'Kashiwa',
    'Kosen1',
    'Ksu',
    'Lasarsat',
    'Ledsat',
    'Lightsail2',
    'Lucky7',
    'Meznsat',
    'Minxss',
    'Mitee1',
    'Mirsat1',
    'Mxl',
    'Mysat',
    'Netsat',
    'Neudose',
    'Neutron1',
    'No44',
    'Nutsat1',
    'Opssat1',
    'Oresat0',
    'Oresat05',
    'Origamisat1',
    'Painani',
    'Pegasus',
    'Picsat',
    'Planetum1',
    'Polyitan1',
    'Pwsat2',
    'Qarman',
    'Qbee',
    'Qube',
    'Qubik',
    'Quetzal1',
    'Ramsat',
    'Randev',
    'Rhoksat',
    'Roseycubesat1',
    'Sakura',
    'Salsat',
    'Sanosat1',
    'Selfiesat',
    'Sharjahsat1',
    'Siriussat',
    'Skcube',
    'Snet',
    'Spoc',
    'Sputnixusp',
    'Sr0',
    'Strand',
    'Stratosattk1',
    'Suchai2',
    'Targit',
    'Us6',
    'Uwe4',
    'Veronika',
    'Vzlusat2',
    'Yomogi',
]

FIELD_REGEX = re.compile(
    r':field (?P<field>[\*\w]+): (?P<attribute>.*?)'
    r'(?:(?=:field)|\Z|\n)', re.S)

UNKNOWN_SIZE_NOTATION = '___'


def get_attribute(obj, name):
    """
    Get element by index in case of list
    Get attribute of object by namein case of non-list object
    """
    if isinstance(obj, list):
        try:
            return obj[int(name)]
        except ValueError:
            return getattr(obj, name).name
    if isinstance(getattr(obj, name), enum.Enum):
        return getattr(obj, name).name
    return getattr(obj, name)


def get_dynamic_fields(obj, path_list, indexes, inframe_index, key):
    """
    Get element by recusion in case of unknown sized list
    """
    if inframe_index == len(path_list):
        pos = 1
        for k in indexes:
            key = key.replace(UNKNOWN_SIZE_NOTATION,
                              '_' + str(indexes[k][0] - 1) + '_', pos)
            pos += 1

        return obj, key

    name = path_list[inframe_index]
    if isinstance(obj, list):
        try:
            if name == UNKNOWN_SIZE_NOTATION:
                index_key = indexes[path_list[inframe_index - 1]][0]
                indexes[path_list[inframe_index - 1]][0] += 1
                if indexes[path_list[inframe_index - 1]][1] is None:
                    indexes[path_list[inframe_index - 1]][1] = len(obj)
                return get_dynamic_fields(obj[index_key], path_list, indexes,
                                          inframe_index + 1, key)

            return get_dynamic_fields(obj[int(name)], path_list, indexes,
                                      inframe_index + 1, key)
        except ValueError:
            return get_dynamic_fields(
                getattr(obj, name).name, path_list, indexes, inframe_index + 1,
                key)
    if isinstance(getattr(obj, name), enum.Enum):
        return get_dynamic_fields(
            getattr(obj, name).name, path_list, indexes, inframe_index + 1,
            key)
    return get_dynamic_fields(getattr(obj, name), path_list, indexes,
                              inframe_index + 1, key)


def get_fields(struct, empty=False):
    """
    Get fields defined in docstring
    """
    fields = {}
    dynamic_fields = {}

    try:
        doc_fields = FIELD_REGEX.findall(struct.__doc__)
    except TypeError:
        return fields

    for key, value in doc_fields:
        try:
            if UNKNOWN_SIZE_NOTATION not in key:
                fields[key] = functools.reduce(get_attribute, value.split('.'),
                                               struct)
            else:
                key_values = key.split(UNKNOWN_SIZE_NOTATION)
                for i in range(0, len(key_values) - 1):
                    dynamic_fields[key_values[i]] = [0, None]

                path_list = value.split('.')
                while True:
                    value, generated_key = get_dynamic_fields(
                        struct, path_list, dynamic_fields, 0, key)
                    fields[generated_key] = value
                    first_dynamic_field_key = next(iter(dynamic_fields))
                    if dynamic_fields[first_dynamic_field_key][
                            0] == dynamic_fields[first_dynamic_field_key][1]:
                        break

        except AttributeError:
            if empty:
                fields[key] = None

    return fields



# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Geoscan(KaitaiStruct):
    """:field callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field ssid_mask: ax25_frame.ax25_header.dest_ssid_raw.ssid_mask
    :field ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field src_callsign_raw_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid_raw_ssid_mask: ax25_frame.ax25_header.src_ssid_raw.ssid_mask
    :field src_ssid_raw_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.ax25_header.pid
    :field beacon_id: ax25_frame.payload.beacon_id
    :field eps_timestamp: ax25_frame.payload.eps_timestamp
    :field eps_mode: ax25_frame.payload.eps_mode
    :field eps_switch_count: ax25_frame.payload.eps_switch_count
    :field eps_consumption_current: ax25_frame.payload.eps_consumption_current
    :field eps_system_current: ax25_frame.payload.eps_system_current
    :field eps_cell_voltage_half: ax25_frame.payload.eps_cell_voltage_half
    :field eps_cell_voltage_full: ax25_frame.payload.eps_cell_voltage_full
    :field eps_systems_status: ax25_frame.payload.eps_systems_status
    :field eps_temperature_cell1: ax25_frame.payload.eps_temperature_cell1
    :field eps_temperature_cell2: ax25_frame.payload.eps_temperature_cell2
    :field eps_boot_count: ax25_frame.payload.eps_boot_count
    :field eps_heater_mode: ax25_frame.payload.eps_heater_mode
    :field eps_reserved: ax25_frame.payload.eps_reserved
    :field obc_boot_count: ax25_frame.payload.obc_boot_count
    :field obc_active_status: ax25_frame.payload.obc_active_status
    :field obc_temperature_pos_x: ax25_frame.payload.obc_temperature_pos_x
    :field obc_temperature_neg_x: ax25_frame.payload.obc_temperature_neg_x
    :field obc_temperature_pos_y: ax25_frame.payload.obc_temperature_pos_y
    :field obc_temperature_neg_y: ax25_frame.payload.obc_temperature_neg_y
    :field gnss_sat_number: ax25_frame.payload.gnss_sat_number
    :field adcs_mode: ax25_frame.payload.adcs_mode
    :field adcs_reserved: ax25_frame.payload.adcs_reserved
    :field cam_photos_number: ax25_frame.payload.cam_photos_number
    :field cam_mode: ax25_frame.payload.cam_mode
    :field cam_reserved: ax25_frame.payload.cam_reserved
    :field comm_type: ax25_frame.payload.comm_type
    :field comm_bus_voltage: ax25_frame.payload.comm_bus_voltage
    :field comm_boot_count: ax25_frame.payload.comm_boot_count
    :field comm_rssi: ax25_frame.payload.comm_rssi
    :field comm_rssi_minimal: ax25_frame.payload.comm_rssi_minimal
    :field comm_received_valid_packets: ax25_frame.payload.comm_received_valid_packets
    :field comm_received_invalid_packets: ax25_frame.payload.comm_received_invalid_packets
    :field comm_sent_packets: ax25_frame.payload.comm_sent_packets
    :field comm_status: ax25_frame.payload.comm_status
    :field comm_mode: ax25_frame.payload.comm_mode
    :field comm_temperature: ax25_frame.payload.comm_temperature
    :field comm_qso_received: ax25_frame.payload.comm_qso_received
    :field comm_reserved: ax25_frame.payload.comm_reserved
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Geoscan.Ax25Frame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Geoscan.Ax25Header(self._io, self, self._root)
            self.payload = Geoscan.GeoscanBeaconTlm(self._io, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Geoscan.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Geoscan.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Geoscan.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Geoscan.SsidMask(self._io, self, self._root)
            self.ctl = self._io.read_u1()
            self.pid = self._io.read_u1()


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return getattr(self, '_m_ssid', None)


    class GeoscanBeaconTlm(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.beacon_id = self._io.read_u1()
            self.eps_timestamp = self._io.read_u4le()
            self.eps_mode = self._io.read_u1()
            self.eps_switch_count = self._io.read_u1()
            self.eps_consumption_current = self._io.read_u2le()
            self.eps_solar_cells_current = self._io.read_u2le()
            self.eps_cell_voltage_half = self._io.read_u2le()
            self.eps_cell_voltage_full = self._io.read_u2le()
            self.eps_systems_status = self._io.read_u2le()
            self.eps_temperature_cell1 = self._io.read_s1()
            self.eps_temperature_cell2 = self._io.read_s1()
            self.eps_boot_count = self._io.read_u2le()
            self.eps_heater_mode = self._io.read_u1()
            self.eps_reserved = self._io.read_u2le()
            self.obc_boot_count = self._io.read_u2le()
            self.obc_active_status = self._io.read_u1()
            self.obc_temperature_pos_x = self._io.read_s1()
            self.obc_temperature_neg_x = self._io.read_s1()
            self.obc_temperature_pos_y = self._io.read_s1()
            self.obc_temperature_neg_y = self._io.read_s1()
            self.gnss_sat_number = self._io.read_u1()
            self.adcs_mode = self._io.read_u1()
            self.adcs_reserved = self._io.read_u1()
            self.cam_photos_number = self._io.read_u1()
            self.cam_mode = self._io.read_u1()
            self.cam_reserved = self._io.read_u4le()
            self.comm_type = self._io.read_u1()
            self.comm_bus_voltage = self._io.read_u2le()
            self.comm_boot_count = self._io.read_u2le()
            self.comm_rssi = self._io.read_s1()
            self.comm_rssi_minimal = self._io.read_s1()
            self.comm_received_valid_packets = self._io.read_u1()
            self.comm_received_invalid_packets = self._io.read_u1()
            self.comm_sent_packets = self._io.read_u1()
            self.comm_status = self._io.read_u1()
            self.comm_mode = self._io.read_u1()
            self.comm_temperature = self._io.read_s1()
            self.comm_qso_received = self._io.read_u1()
            self.comm_reserved = self._io.read_u2le()


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Geoscan.Callsign(_io__raw_callsign_ror, self, self._root)






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Sputnixusp(KaitaiStruct):
    """:field callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field ssid_mask: ax25_frame.ax25_header.dest_ssid_raw.ssid_mask
    :field ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field src_callsign_raw_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid_raw_ssid_mask: ax25_frame.ax25_header.src_ssid_raw.ssid_mask
    :field src_ssid_raw_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.ax25_header.pid
    :field packet_type: ax25_frame.payload.packet_type
    :field skip1: ax25_frame.payload.tlm.skip1
    :field skip2: ax25_frame.payload.tlm.skip2
    :field skip3: ax25_frame.payload.tlm.skip3
    :field usb1: ax25_frame.payload.tlm.usb1
    :field usb2: ax25_frame.payload.tlm.usb2
    :field usb3: ax25_frame.payload.tlm.usb3
    :field isb1: ax25_frame.payload.tlm.isb1
    :field isb2: ax25_frame.payload.tlm.isb2
    :field isb3: ax25_frame.payload.tlm.isb3
    :field iab: ax25_frame.payload.tlm.iab
    :field ich1: ax25_frame.payload.tlm.ich1
    :field ich2: ax25_frame.payload.tlm.ich2
    :field ich3: ax25_frame.payload.tlm.ich3
    :field ich4: ax25_frame.payload.tlm.ich4
    :field t1_pw: ax25_frame.payload.tlm.t1_pw
    :field t2_pw: ax25_frame.payload.tlm.t2_pw
    :field t3_pw: ax25_frame.payload.tlm.t3_pw
    :field t4_pw: ax25_frame.payload.tlm.t4_pw
    :field flags1: ax25_frame.payload.tlm.flags1
    :field flags2: ax25_frame.payload.tlm.flags2
    :field flags3: ax25_frame.payload.tlm.flags3
    :field reserved1: ax25_frame.payload.tlm.reserved1
    :field uab: ax25_frame.payload.tlm.uab
    :field reg_tel_id: ax25_frame.payload.tlm.reg_tel_id
    :field time: ax25_frame.payload.tlm.time
    :field nres_ps: ax25_frame.payload.tlm.nres_ps
    :field fl_ps: ax25_frame.payload.tlm.fl_ps
    :field t_amp: ax25_frame.payload.tlm.t_amp
    :field t_uhf: ax25_frame.payload.tlm.t_uhf
    :field rssi_rx: ax25_frame.payload.tlm.rssi_rx
    :field pf: ax25_frame.payload.tlm.pf
    :field pb: ax25_frame.payload.tlm.pb
    :field nres_uhf: ax25_frame.payload.tlm.nres_uhf
    :field fl_uhf: ax25_frame.payload.tlm.fl_uhf
    :field time_uhf: ax25_frame.payload.tlm.time_uhf
    :field uptime_uhf: ax25_frame.payload.tlm.uptime_uhf
    :field current_uhf: ax25_frame.payload.tlm.current_uhf
    :field uuhf: ax25_frame.payload.tlm.uuhf
    :field t_mb: ax25_frame.payload.tlm.t_mb
    :field mx: ax25_frame.payload.tlm.mx
    :field my: ax25_frame.payload.tlm.my
    :field mz: ax25_frame.payload.tlm.mz
    :field vx: ax25_frame.payload.tlm.vx
    :field vy: ax25_frame.payload.tlm.vy
    :field vz: ax25_frame.payload.tlm.vz
    :field nres_ext: ax25_frame.payload.tlm.nres_ext
    :field rcon: ax25_frame.payload.tlm.rcon
    :field fl_ext: ax25_frame.payload.tlm.fl_ext
    :field time_ext: ax25_frame.payload.tlm.time_ext
    :field fl_payload: ax25_frame.payload.tlm.fl_payload
    :field time_send: ax25_frame.payload.tlm.time_send
    :field t_plate: ax25_frame.payload.tlm.t_plate
    :field t_cpu: ax25_frame.payload.tlm.t_cpu
    :field cursens1: ax25_frame.payload.tlm.cursens1
    :field cursens2: ax25_frame.payload.tlm.cursens2
    :field nrst: ax25_frame.payload.tlm.nrst
    :field time_rst: ax25_frame.payload.tlm.time_rst
    :field ch1rate: ax25_frame.payload.tlm.ch1rate
    :field ch2rate: ax25_frame.payload.tlm.ch2rate
    :field ch3rate: ax25_frame.payload.tlm.ch3rate
    :field ch4rate: ax25_frame.payload.tlm.ch4rate
    :field ch5rate: ax25_frame.payload.tlm.ch5rate
    :field ch6rate: ax25_frame.payload.tlm.ch6rate
    :field ptrend1: ax25_frame.payload.tlm.ptrend1
    :field ptrctl1: ax25_frame.payload.tlm.ptrctl1
    :field ptrend2: ax25_frame.payload.tlm.ptrend2
    :field ptrctl2: ax25_frame.payload.tlm.ptrctl2
    :field ptrend3: ax25_frame.payload.tlm.ptrend3
    :field ptrctl3: ax25_frame.payload.tlm.ptrctl3
    :field lastevent_ch1_1: ax25_frame.payload.tlm.lastevent_ch1_1
    :field lastevent_ch1_2: ax25_frame.payload.tlm.lastevent_ch1_2
    :field lastevent_ch1_3: ax25_frame.payload.tlm.lastevent_ch1_3
    :field lastevent_ch2_1: ax25_frame.payload.tlm.lastevent_ch2_1
    :field lastevent_ch2_2: ax25_frame.payload.tlm.lastevent_ch2_2
    :field lastevent_ch2_3: ax25_frame.payload.tlm.lastevent_ch2_3
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Sputnixusp.Ax25Frame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Sputnixusp.Ax25Header(self._io, self, self._root)
            self.payload = Sputnixusp.BeaconTlm(self._io, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Sputnixusp.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Sputnixusp.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Sputnixusp.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Sputnixusp.SsidMask(self._io, self, self._root)
            self.ctl = self._io.read_u1()
            self.pid = self._io.read_u1()


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


    class ExtendedTlm(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.t_mb = self._io.read_u2le()
            self.mx = self._io.read_u4le()
            self.my = self._io.read_u4le()
            self.mz = self._io.read_u4le()
            self.vx = self._io.read_u4le()
            self.vy = self._io.read_u4le()
            self.vz = self._io.read_u4le()
            self.nres_ext = self._io.read_u1()
            self.rcon = self._io.read_u1()
            self.fl_ext = self._io.read_u1()
            self.time_ext = self._io.read_u4le()
            self.fl_payload = self._io.read_u1()
            self.time_send = self._io.read_u4le()
            self.t_plate = self._io.read_u2le()
            self.t_cpu = self._io.read_u2le()
            self.cursens1 = self._io.read_u2le()
            self.cursens2 = self._io.read_u2le()
            self.nrst = self._io.read_u1()
            self.time_rst = self._io.read_u4le()
            self.ch1rate = self._io.read_u2le()
            self.ch2rate = self._io.read_u2le()
            self.ch3rate = self._io.read_u2le()
            self.ch4rate = self._io.read_u2le()
            self.ch5rate = self._io.read_u2le()
            self.ch6rate = self._io.read_u2le()
            self.ptrend1 = self._io.read_u2le()
            self.ptrctl1 = self._io.read_u2le()
            self.ptrend2 = self._io.read_u2le()
            self.ptrctl2 = self._io.read_u2le()
            self.ptrend3 = self._io.read_u2le()
            self.ptrctl3 = self._io.read_u2le()
            self.lastevent_ch1_1 = self._io.read_u1()
            self.lastevent_ch1_2 = self._io.read_u1()
            self.lastevent_ch1_3 = self._io.read_u1()
            self.lastevent_ch2_1 = self._io.read_u1()
            self.lastevent_ch2_2 = self._io.read_u1()
            self.lastevent_ch2_3 = self._io.read_u1()


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return getattr(self, '_m_ssid', None)


    class BeaconTlm(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.packet_type = self._io.read_u2le()
            _on = self.packet_type
            if _on == 16918:
                self.tlm = Sputnixusp.GeneralTlm(self._io, self, self._root)
            elif _on == 16919:
                self.tlm = Sputnixusp.ExtendedTlm(self._io, self, self._root)


    class GeneralTlm(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.skip1 = self._io.read_u2le()
            self.skip2 = self._io.read_u2le()
            self.skip3 = self._io.read_u2le()
            self.usb1 = self._io.read_u2le()
            self.usb2 = self._io.read_u2le()
            self.usb3 = self._io.read_u2le()
            self.isb1 = self._io.read_u2le()
            self.isb2 = self._io.read_u2le()
            self.isb3 = self._io.read_u2le()
            self.iab = self._io.read_u2le()
            self.ich1 = self._io.read_u2le()
            self.ich2 = self._io.read_u2le()
            self.ich3 = self._io.read_u2le()
            self.ich4 = self._io.read_u2le()
            self.t1_pw = self._io.read_u2le()
            self.t2_pw = self._io.read_u2le()
            self.t3_pw = self._io.read_u2le()
            self.t4_pw = self._io.read_u2le()
            self.flags1 = self._io.read_u1()
            self.flags2 = self._io.read_u1()
            self.flags3 = self._io.read_u1()
            self.reserved1 = self._io.read_u1()
            self.uab = self._io.read_u2le()
            self.reg_tel_id = self._io.read_u4le()
            self.time = self._io.read_u4le()
            self.nres_ps = self._io.read_u1()
            self.fl_ps = self._io.read_u1()
            self.t_amp = self._io.read_u1()
            self.t_uhf = self._io.read_u1()
            self.rssi_rx = self._io.read_s2be()
            self.pf = self._io.read_u1()
            self.pb = self._io.read_u1()
            self.nres_uhf = self._io.read_u1()
            self.fl_uhf = self._io.read_u1()
            self.time_uhf = self._io.read_u4le()
            self.uptime_uhf = self._io.read_u4le()
            self.current_uhf = self._io.read_u2le()
            self.uuhf = self._io.read_u2le()


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Sputnixusp.Callsign(_io__raw_callsign_ror, self, self._root)






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Dhabisat(KaitaiStruct):
    """:field dest_callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field rpt_callsign: ax25_frame.ax25_header.repeater.rpt_instance[0].rpt_callsign_raw.callsign_ror.callsign
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.payload.pid
    :field data_type: ax25_frame.payload.ax25_info.dhabisat_header.data_type
    :field time_stamp: ax25_frame.payload.ax25_info.dhabisat_header.time_stamp
    :field file_index: ax25_frame.payload.ax25_info.dhabisat_header.file_index
    :field interval: ax25_frame.payload.ax25_info.dhabisat_header.interval
    :field total_packets: ax25_frame.payload.ax25_info.dhabisat_header.total_packets
    :field current_packet: ax25_frame.payload.ax25_info.dhabisat_header.current_packet
    :field payload_ax25_info_beacon_type_callsign: ax25_frame.payload.ax25_info.beacon_type.callsign
    :field obc_mode: ax25_frame.payload.ax25_info.beacon_type.obc_mode
    :field obc_reset_count: ax25_frame.payload.ax25_info.beacon_type.obc_reset_count
    :field obc_timestamp: ax25_frame.payload.ax25_info.beacon_type.obc_timestamp
    :field obc_uptime: ax25_frame.payload.ax25_info.beacon_type.obc_uptime
    :field subsystem_safety_criteria: ax25_frame.payload.ax25_info.beacon_type.subsystem_safety_criteria
    :field telemetry_distribution_counter: ax25_frame.payload.ax25_info.beacon_type.telemetry_distribution_counter
    :field obc_temperature: ax25_frame.payload.ax25_info.beacon_type.obc_temperature
    :field camera_voltage: ax25_frame.payload.ax25_info.beacon_type.camera_voltage
    :field camera_current: ax25_frame.payload.ax25_info.beacon_type.camera_current
    :field battery_temperature: ax25_frame.payload.ax25_info.beacon_type.battery_temperature
    :field battery_voltage: ax25_frame.payload.ax25_info.beacon_type.battery_voltage
    :field adcs_5v_voltage: ax25_frame.payload.ax25_info.beacon_type.adcs_5v_voltage
    :field adcs_5v_current: ax25_frame.payload.ax25_info.beacon_type.adcs_5v_current
    :field adcs_5v_power: ax25_frame.payload.ax25_info.beacon_type.adcs_5v_power
    :field adcs_3v3_voltage: ax25_frame.payload.ax25_info.beacon_type.adcs_3v3_voltage
    :field adcs_3v3_current: ax25_frame.payload.ax25_info.beacon_type.adcs_3v3_current
    :field adcs_3v3_power: ax25_frame.payload.ax25_info.beacon_type.adcs_3v3_power
    :field eps_mode: ax25_frame.payload.ax25_info.beacon_type.eps_mode
    :field eps_reset_cause: ax25_frame.payload.ax25_info.beacon_type.eps_reset_cause
    :field eps_uptime: ax25_frame.payload.ax25_info.beacon_type.eps_uptime
    :field eps_error: ax25_frame.payload.ax25_info.beacon_type.eps_error
    :field eps_system_reset_counter_power_on: ax25_frame.payload.ax25_info.beacon_type.eps_system_reset_counter_power_on
    :field eps_system_reset_counter_watchdog: ax25_frame.payload.ax25_info.beacon_type.eps_system_reset_counter_watchdog
    :field eps_system_reset_counter_commanded: ax25_frame.payload.ax25_info.beacon_type.eps_system_reset_counter_commanded
    :field eps_system_reset_counter_controller: ax25_frame.payload.ax25_info.beacon_type.eps_system_reset_counter_controller
    :field eps_system_reset_counter_low_power: ax25_frame.payload.ax25_info.beacon_type.eps_system_reset_counter_low_power
    :field panel_xp_temperature: ax25_frame.payload.ax25_info.beacon_type.panel_xp_temperature
    :field panel_xn_temperature: ax25_frame.payload.ax25_info.beacon_type.panel_xn_temperature
    :field panel_yp_temperature: ax25_frame.payload.ax25_info.beacon_type.panel_yp_temperature
    :field panel_yn_temperature: ax25_frame.payload.ax25_info.beacon_type.panel_yn_temperature
    :field panel_zp_temperature: ax25_frame.payload.ax25_info.beacon_type.panel_zp_temperature
    :field panel_zn_temperature: ax25_frame.payload.ax25_info.beacon_type.panel_zn_temperature
    :field tx_pa_temp: ax25_frame.payload.ax25_info.beacon_type.tx_pa_temp
    :field attitude_estimation_mode: ax25_frame.payload.ax25_info.beacon_type.attitude_estimation_mode
    :field attitude_control_mode: ax25_frame.payload.ax25_info.beacon_type.attitude_control_mode
    :field adcs_run_mode: ax25_frame.payload.ax25_info.beacon_type.adcs_run_mode
    :field asgp4_mode: ax25_frame.payload.ax25_info.beacon_type.asgp4_mode
    :field cubecontrol_signal_enabled: ax25_frame.payload.ax25_info.beacon_type.cubecontrol_signal_enabled
    :field cubecontrol_motor_enabled: ax25_frame.payload.ax25_info.beacon_type.cubecontrol_motor_enabled
    :field cubesense1_enabled: ax25_frame.payload.ax25_info.beacon_type.cubesense1_enabled
    :field cubesense2_enabled: ax25_frame.payload.ax25_info.beacon_type.cubesense2_enabled
    :field cubewheel1_enabled: ax25_frame.payload.ax25_info.beacon_type.cubewheel1_enabled
    :field cubewheel2_enabled: ax25_frame.payload.ax25_info.beacon_type.cubewheel2_enabled
    :field cubewheel3_enabled: ax25_frame.payload.ax25_info.beacon_type.cubewheel3_enabled
    :field cubestar_enabled: ax25_frame.payload.ax25_info.beacon_type.cubestar_enabled
    :field gps_receiver_enabled: ax25_frame.payload.ax25_info.beacon_type.gps_receiver_enabled
    :field gps_lna_power_enabled: ax25_frame.payload.ax25_info.beacon_type.gps_lna_power_enabled
    :field motor_driver_enabled: ax25_frame.payload.ax25_info.beacon_type.motor_driver_enabled
    :field sun_is_above_local_horizon: ax25_frame.payload.ax25_info.beacon_type.sun_is_above_local_horizon
    :field cubesense1_communications_error: ax25_frame.payload.ax25_info.beacon_type.cubesense1_communications_error
    :field cubesense2_communications_error: ax25_frame.payload.ax25_info.beacon_type.cubesense2_communications_error
    :field cubecontrol_signal_communications_error: ax25_frame.payload.ax25_info.beacon_type.cubecontrol_signal_communications_error
    :field cubecontrol_motor_communications_error: ax25_frame.payload.ax25_info.beacon_type.cubecontrol_motor_communications_error
    :field cubewheel1_communications_error: ax25_frame.payload.ax25_info.beacon_type.cubewheel1_communications_error
    :field cubewheel2_communications_error: ax25_frame.payload.ax25_info.beacon_type.cubewheel2_communications_error
    :field cubewheel3_communications_error: ax25_frame.payload.ax25_info.beacon_type.cubewheel3_communications_error
    :field cubestar_communications_error: ax25_frame.payload.ax25_info.beacon_type.cubestar_communications_error
    :field magnetometer_range_error: ax25_frame.payload.ax25_info.beacon_type.magnetometer_range_error
    :field cam1_sram_overcurrent_detected: ax25_frame.payload.ax25_info.beacon_type.cam1_sram_overcurrent_detected
    :field cam1_3v3_overcurrent_detected: ax25_frame.payload.ax25_info.beacon_type.cam1_3v3_overcurrent_detected
    :field cam1_sensor_busy_error: ax25_frame.payload.ax25_info.beacon_type.cam1_sensor_busy_error
    :field cam1_sensor_detection_error: ax25_frame.payload.ax25_info.beacon_type.cam1_sensor_detection_error
    :field sun_sensor_range_error: ax25_frame.payload.ax25_info.beacon_type.sun_sensor_range_error
    :field cam2_sram_overcurrent_detected: ax25_frame.payload.ax25_info.beacon_type.cam2_sram_overcurrent_detected
    :field cam2_3v3_overcurrent_detected: ax25_frame.payload.ax25_info.beacon_type.cam2_3v3_overcurrent_detected
    :field cam2_sensor_busy_error: ax25_frame.payload.ax25_info.beacon_type.cam2_sensor_busy_error
    :field cam2_sensor_detection_error: ax25_frame.payload.ax25_info.beacon_type.cam2_sensor_detection_error
    :field nadir_sensor_range_error: ax25_frame.payload.ax25_info.beacon_type.nadir_sensor_range_error
    :field rate_sensor_range_error: ax25_frame.payload.ax25_info.beacon_type.rate_sensor_range_error
    :field wheel_speed_range_error: ax25_frame.payload.ax25_info.beacon_type.wheel_speed_range_error
    :field coarse_sun_sensor_error: ax25_frame.payload.ax25_info.beacon_type.coarse_sun_sensor_error
    :field startracker_match_error: ax25_frame.payload.ax25_info.beacon_type.startracker_match_error
    :field startracker_overcurrent_detected: ax25_frame.payload.ax25_info.beacon_type.startracker_overcurrent_detected
    :field orbit_parameters_are_invalid: ax25_frame.payload.ax25_info.beacon_type.orbit_parameters_are_invalid
    :field configuration_is_invalid: ax25_frame.payload.ax25_info.beacon_type.configuration_is_invalid
    :field control_mode_change_is_not_allowed: ax25_frame.payload.ax25_info.beacon_type.control_mode_change_is_not_allowed
    :field estimator_change_is_not_allowed: ax25_frame.payload.ax25_info.beacon_type.estimator_change_is_not_allowed
    :field current_magnetometer_sampling_mode: ax25_frame.payload.ax25_info.beacon_type.current_magnetometer_sampling_mode
    :field modelled_and_measured_magnetic_field_differs_in_size: ax25_frame.payload.ax25_info.beacon_type.modelled_and_measured_magnetic_field_differs_in_size
    :field node_recovery_error: ax25_frame.payload.ax25_info.beacon_type.node_recovery_error
    :field cubesense1_runtime_error: ax25_frame.payload.ax25_info.beacon_type.cubesense1_runtime_error
    :field cubesense2_runtime_error: ax25_frame.payload.ax25_info.beacon_type.cubesense2_runtime_error
    :field cubecontrol_signal_runtime_error: ax25_frame.payload.ax25_info.beacon_type.cubecontrol_signal_runtime_error
    :field cubecontrol_motor_runtime_error: ax25_frame.payload.ax25_info.beacon_type.cubecontrol_motor_runtime_error
    :field cubewheel1_runtime_error: ax25_frame.payload.ax25_info.beacon_type.cubewheel1_runtime_error
    :field cubewheel2_runtime_error: ax25_frame.payload.ax25_info.beacon_type.cubewheel2_runtime_error
    :field cubewheel3_runtime_error: ax25_frame.payload.ax25_info.beacon_type.cubewheel3_runtime_error
    :field cubestar_runtime_error: ax25_frame.payload.ax25_info.beacon_type.cubestar_runtime_error
    :field magnetometer_error: ax25_frame.payload.ax25_info.beacon_type.magnetometer_error
    :field rate_sensor_failure: ax25_frame.payload.ax25_info.beacon_type.rate_sensor_failure
    :field padding_1: ax25_frame.payload.ax25_info.beacon_type.padding_1
    :field padding_2: ax25_frame.payload.ax25_info.beacon_type.padding_2
    :field padding_3: ax25_frame.payload.ax25_info.beacon_type.padding_3
    :field estimated_roll_angle: ax25_frame.payload.ax25_info.beacon_type.estimated_roll_angle
    :field estimated_pitch_angle: ax25_frame.payload.ax25_info.beacon_type.estimated_pitch_angle
    :field estimated_yaw_angle: ax25_frame.payload.ax25_info.beacon_type.estimated_yaw_angle
    :field estimated_q1: ax25_frame.payload.ax25_info.beacon_type.estimated_q1
    :field estimated_q2: ax25_frame.payload.ax25_info.beacon_type.estimated_q2
    :field estimated_q3: ax25_frame.payload.ax25_info.beacon_type.estimated_q3
    :field estimated_x_angular_rate: ax25_frame.payload.ax25_info.beacon_type.estimated_x_angular_rate
    :field estimated_y_angular_rate: ax25_frame.payload.ax25_info.beacon_type.estimated_y_angular_rate
    :field estimated_z_angular_rate: ax25_frame.payload.ax25_info.beacon_type.estimated_z_angular_rate
    :field x_position: ax25_frame.payload.ax25_info.beacon_type.x_position
    :field y_position: ax25_frame.payload.ax25_info.beacon_type.y_position
    :field z_position: ax25_frame.payload.ax25_info.beacon_type.z_position
    :field x_velocity: ax25_frame.payload.ax25_info.beacon_type.x_velocity
    :field y_velocity: ax25_frame.payload.ax25_info.beacon_type.y_velocity
    :field z_velocity: ax25_frame.payload.ax25_info.beacon_type.z_velocity
    :field latitude: ax25_frame.payload.ax25_info.beacon_type.latitude
    :field longitude: ax25_frame.payload.ax25_info.beacon_type.longitude
    :field altitude: ax25_frame.payload.ax25_info.beacon_type.altitude
    :field ecef_position_x: ax25_frame.payload.ax25_info.beacon_type.ecef_position_x
    :field ecef_position_y: ax25_frame.payload.ax25_info.beacon_type.ecef_position_y
    :field ecef_position_z: ax25_frame.payload.ax25_info.beacon_type.ecef_position_z
    :field cubesense1_3v3_current: ax25_frame.payload.ax25_info.beacon_type.cubesense1_3v3_current
    :field cubesense1_cam_sram_current: ax25_frame.payload.ax25_info.beacon_type.cubesense1_cam_sram_current
    :field cubesense2_3v3_current: ax25_frame.payload.ax25_info.beacon_type.cubesense2_3v3_current
    :field cubesense2_cam_sram_current: ax25_frame.payload.ax25_info.beacon_type.cubesense2_cam_sram_current
    :field cubecontrol_3v3_current: ax25_frame.payload.ax25_info.beacon_type.cubecontrol_3v3_current
    :field cubecontrol_5v_current: ax25_frame.payload.ax25_info.beacon_type.cubecontrol_5v_current
    :field cubecontrol_vbat_current: ax25_frame.payload.ax25_info.beacon_type.cubecontrol_vbat_current
    :field wheel1current: ax25_frame.payload.ax25_info.beacon_type.wheel1current
    :field wheel2current: ax25_frame.payload.ax25_info.beacon_type.wheel2current
    :field wheel3current: ax25_frame.payload.ax25_info.beacon_type.wheel3current
    :field cubestarcurrent: ax25_frame.payload.ax25_info.beacon_type.cubestarcurrent
    :field magnetorquercurrent: ax25_frame.payload.ax25_info.beacon_type.magnetorquercurrent
    :field cubestar_mcu_temperature: ax25_frame.payload.ax25_info.beacon_type.cubestar_mcu_temperature
    :field adcs_mcu_temperature: ax25_frame.payload.ax25_info.beacon_type.adcs_mcu_temperature
    :field magnetometer_temperature: ax25_frame.payload.ax25_info.beacon_type.magnetometer_temperature
    :field redundant_magnetometer_temperature: ax25_frame.payload.ax25_info.beacon_type.redundant_magnetometer_temperature
    :field xrate_sensor_temperature: ax25_frame.payload.ax25_info.beacon_type.xrate_sensor_temperature
    :field yrate_sensor_temperature: ax25_frame.payload.ax25_info.beacon_type.yrate_sensor_temperature
    :field zrate_sensor_temperature: ax25_frame.payload.ax25_info.beacon_type.zrate_sensor_temperature
    :field x_angular_rate: ax25_frame.payload.ax25_info.beacon_type.x_angular_rate
    :field y_angular_rate: ax25_frame.payload.ax25_info.beacon_type.y_angular_rate
    :field z_angular_rate: ax25_frame.payload.ax25_info.beacon_type.z_angular_rate
    :field x_wheelspeed: ax25_frame.payload.ax25_info.beacon_type.x_wheelspeed
    :field y_wheelspeed: ax25_frame.payload.ax25_info.beacon_type.y_wheelspeed
    :field z_wheelspeed: ax25_frame.payload.ax25_info.beacon_type.z_wheelspeed
    :field message: ax25_frame.payload.ax25_info.beacon_type.message
    
    Attention: `rpt_callsign` cannot be accessed because `rpt_instance` is an
    array of unknown size at the beginning of the parsing process! Left an
    example in here.
    
    .. seealso::
       Source - https://gitlab.com/librespacefoundation/satnogs/satnogs-decoders/-/issues/57
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Dhabisat.Ax25Frame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Dhabisat.Ax25Header(self._io, self, self._root)
            _on = (self.ax25_header.ctl & 19)
            if _on == 0:
                self.payload = Dhabisat.IFrame(self._io, self, self._root)
            elif _on == 3:
                self.payload = Dhabisat.UiFrame(self._io, self, self._root)
            elif _on == 19:
                self.payload = Dhabisat.UiFrame(self._io, self, self._root)
            elif _on == 16:
                self.payload = Dhabisat.IFrame(self._io, self, self._root)
            elif _on == 18:
                self.payload = Dhabisat.IFrame(self._io, self, self._root)
            elif _on == 2:
                self.payload = Dhabisat.IFrame(self._io, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Dhabisat.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Dhabisat.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Dhabisat.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Dhabisat.SsidMask(self._io, self, self._root)
            if (self.src_ssid_raw.ssid_mask & 1) == 0:
                self.repeater = Dhabisat.Repeater(self._io, self, self._root)

            self.ctl = self._io.read_u1()


    class UiFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Dhabisat.DhabisatPayload(_io__raw_ax25_info, self, self._root)


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")
            if not  ((self.callsign == u"A68MY ") or (self.callsign == u"A68MX ") or (self.callsign == u"A68KU ") or (self.callsign == u"NOCALL")) :
                raise kaitaistruct.ValidationNotAnyOfError(self.callsign, self._io, u"/types/callsign/seq/0")


    class DhabisatPayload(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dhabisat_header = Dhabisat.DhabisatHeaderT(self._io, self, self._root)
            _on = self._io.size()
            if _on == 187:
                self.beacon_type = Dhabisat.DhabisatBeacon(self._io, self, self._root)
            else:
                self.beacon_type = Dhabisat.DhabisatGenericPacket(self._io, self, self._root)


    class IFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self.ax25_info = self._io.read_bytes_full()


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return getattr(self, '_m_ssid', None)


    class DhabisatGenericPacket(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.message = (self._io.read_bytes_full()).decode(u"utf-8")


    class Repeaters(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rpt_callsign_raw = Dhabisat.CallsignRaw(self._io, self, self._root)
            self.rpt_ssid_raw = Dhabisat.SsidMask(self._io, self, self._root)


    class Repeater(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rpt_instance = []
            i = 0
            while True:
                _ = Dhabisat.Repeaters(self._io, self, self._root)
                self.rpt_instance.append(_)
                if (_.rpt_ssid_raw.ssid_mask & 1) == 1:
                    break
                i += 1


    class DhabisatBeacon(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(5)).decode(u"utf-8")
            self.obc_mode = self._io.read_u1()
            self.obc_reset_count = self._io.read_u4le()
            self.obc_timestamp = self._io.read_u4le()
            self.obc_uptime = self._io.read_u4le()
            self.subsystem_safety_criteria = self._io.read_u1()
            self.telemetry_distribution_counter = self._io.read_u4le()
            self.obc_temperature = self._io.read_u1()
            self.camera_voltage = self._io.read_u1()
            self.camera_current = self._io.read_u2le()
            self.battery_temperature = self._io.read_s2le()
            self.battery_voltage = self._io.read_s2le()
            self.adcs_5v_voltage = self._io.read_u2le()
            self.adcs_5v_current = self._io.read_u2le()
            self.adcs_5v_power = self._io.read_u2le()
            self.adcs_3v3_voltage = self._io.read_u2le()
            self.adcs_3v3_current = self._io.read_u2le()
            self.adcs_3v3_power = self._io.read_u2le()
            self.eps_mode = self._io.read_u1()
            self.eps_reset_cause = self._io.read_u1()
            self.eps_uptime = self._io.read_u4le()
            self.eps_error = self._io.read_u2le()
            self.eps_system_reset_counter_power_on = self._io.read_u2le()
            self.eps_system_reset_counter_watchdog = self._io.read_u2le()
            self.eps_system_reset_counter_commanded = self._io.read_u2le()
            self.eps_system_reset_counter_controller = self._io.read_u2le()
            self.eps_system_reset_counter_low_power = self._io.read_u2le()
            self.panel_xp_temperature = self._io.read_u1()
            self.panel_xn_temperature = self._io.read_u1()
            self.panel_yp_temperature = self._io.read_u1()
            self.panel_yn_temperature = self._io.read_u1()
            self.panel_zp_temperature = self._io.read_u1()
            self.panel_zn_temperature = self._io.read_u1()
            self.tx_pa_temp = self._io.read_s2le()
            self.attitude_estimation_mode = self._io.read_bits_int_le(4)
            self.attitude_control_mode = self._io.read_bits_int_le(4)
            self.adcs_run_mode = self._io.read_bits_int_le(2)
            self.asgp4_mode = self._io.read_bits_int_le(2)
            self.cubecontrol_signal_enabled = self._io.read_bits_int_le(1) != 0
            self.cubecontrol_motor_enabled = self._io.read_bits_int_le(1) != 0
            self.cubesense1_enabled = self._io.read_bits_int_le(1) != 0
            self.cubesense2_enabled = self._io.read_bits_int_le(1) != 0
            self.cubewheel1_enabled = self._io.read_bits_int_le(1) != 0
            self.cubewheel2_enabled = self._io.read_bits_int_le(1) != 0
            self.cubewheel3_enabled = self._io.read_bits_int_le(1) != 0
            self.cubestar_enabled = self._io.read_bits_int_le(1) != 0
            self.gps_receiver_enabled = self._io.read_bits_int_le(1) != 0
            self.gps_lna_power_enabled = self._io.read_bits_int_le(1) != 0
            self.motor_driver_enabled = self._io.read_bits_int_le(1) != 0
            self.sun_is_above_local_horizon = self._io.read_bits_int_le(1) != 0
            self.cubesense1_communications_error = self._io.read_bits_int_le(1) != 0
            self.cubesense2_communications_error = self._io.read_bits_int_le(1) != 0
            self.cubecontrol_signal_communications_error = self._io.read_bits_int_le(1) != 0
            self.cubecontrol_motor_communications_error = self._io.read_bits_int_le(1) != 0
            self.cubewheel1_communications_error = self._io.read_bits_int_le(1) != 0
            self.cubewheel2_communications_error = self._io.read_bits_int_le(1) != 0
            self.cubewheel3_communications_error = self._io.read_bits_int_le(1) != 0
            self.cubestar_communications_error = self._io.read_bits_int_le(1) != 0
            self.magnetometer_range_error = self._io.read_bits_int_le(1) != 0
            self.cam1_sram_overcurrent_detected = self._io.read_bits_int_le(1) != 0
            self.cam1_3v3_overcurrent_detected = self._io.read_bits_int_le(1) != 0
            self.cam1_sensor_busy_error = self._io.read_bits_int_le(1) != 0
            self.cam1_sensor_detection_error = self._io.read_bits_int_le(1) != 0
            self.sun_sensor_range_error = self._io.read_bits_int_le(1) != 0
            self.cam2_sram_overcurrent_detected = self._io.read_bits_int_le(1) != 0
            self.cam2_3v3_overcurrent_detected = self._io.read_bits_int_le(1) != 0
            self.cam2_sensor_busy_error = self._io.read_bits_int_le(1) != 0
            self.cam2_sensor_detection_error = self._io.read_bits_int_le(1) != 0
            self.nadir_sensor_range_error = self._io.read_bits_int_le(1) != 0
            self.rate_sensor_range_error = self._io.read_bits_int_le(1) != 0
            self.wheel_speed_range_error = self._io.read_bits_int_le(1) != 0
            self.coarse_sun_sensor_error = self._io.read_bits_int_le(1) != 0
            self.startracker_match_error = self._io.read_bits_int_le(1) != 0
            self.startracker_overcurrent_detected = self._io.read_bits_int_le(1) != 0
            self.orbit_parameters_are_invalid = self._io.read_bits_int_le(1) != 0
            self.configuration_is_invalid = self._io.read_bits_int_le(1) != 0
            self.control_mode_change_is_not_allowed = self._io.read_bits_int_le(1) != 0
            self.estimator_change_is_not_allowed = self._io.read_bits_int_le(1) != 0
            self.current_magnetometer_sampling_mode = self._io.read_bits_int_le(2)
            self.modelled_and_measured_magnetic_field_differs_in_size = self._io.read_bits_int_le(1) != 0
            self.node_recovery_error = self._io.read_bits_int_le(1) != 0
            self.cubesense1_runtime_error = self._io.read_bits_int_le(1) != 0
            self.cubesense2_runtime_error = self._io.read_bits_int_le(1) != 0
            self.cubecontrol_signal_runtime_error = self._io.read_bits_int_le(1) != 0
            self.cubecontrol_motor_runtime_error = self._io.read_bits_int_le(1) != 0
            self.cubewheel1_runtime_error = self._io.read_bits_int_le(1) != 0
            self.cubewheel2_runtime_error = self._io.read_bits_int_le(1) != 0
            self.cubewheel3_runtime_error = self._io.read_bits_int_le(1) != 0
            self.cubestar_runtime_error = self._io.read_bits_int_le(1) != 0
            self.magnetometer_error = self._io.read_bits_int_le(1) != 0
            self.rate_sensor_failure = self._io.read_bits_int_le(1) != 0
            self.padding_1 = self._io.read_bits_int_le(6)
            self._io.align_to_byte()
            self.padding_2 = self._io.read_u2le()
            self.padding_3 = self._io.read_u1()
            self.estimated_roll_angle = self._io.read_s2le()
            self.estimated_pitch_angle = self._io.read_s2le()
            self.estimated_yaw_angle = self._io.read_s2le()
            self.estimated_q1 = self._io.read_s2le()
            self.estimated_q2 = self._io.read_s2le()
            self.estimated_q3 = self._io.read_s2le()
            self.estimated_x_angular_rate = self._io.read_s2le()
            self.estimated_y_angular_rate = self._io.read_s2le()
            self.estimated_z_angular_rate = self._io.read_s2le()
            self.x_position = self._io.read_s2le()
            self.y_position = self._io.read_s2le()
            self.z_position = self._io.read_s2le()
            self.x_velocity = self._io.read_s2le()
            self.y_velocity = self._io.read_s2le()
            self.z_velocity = self._io.read_s2le()
            self.latitude = self._io.read_s2le()
            self.longitude = self._io.read_s2le()
            self.altitude = self._io.read_s2le()
            self.ecef_position_x = self._io.read_s2le()
            self.ecef_position_y = self._io.read_s2le()
            self.ecef_position_z = self._io.read_s2le()
            self.cubesense1_3v3_current = self._io.read_u2le()
            self.cubesense1_cam_sram_current = self._io.read_u2le()
            self.cubesense2_3v3_current = self._io.read_u2le()
            self.cubesense2_cam_sram_current = self._io.read_u2le()
            self.cubecontrol_3v3_current = self._io.read_u2le()
            self.cubecontrol_5v_current = self._io.read_u2le()
            self.cubecontrol_vbat_current = self._io.read_u2le()
            self.wheel1current = self._io.read_u2le()
            self.wheel2current = self._io.read_u2le()
            self.wheel3current = self._io.read_u2le()
            self.cubestarcurrent = self._io.read_u2le()
            self.magnetorquercurrent = self._io.read_u2le()
            self.cubestar_mcu_temperature = self._io.read_s2le()
            self.adcs_mcu_temperature = self._io.read_s2le()
            self.magnetometer_temperature = self._io.read_s2le()
            self.redundant_magnetometer_temperature = self._io.read_s2le()
            self.xrate_sensor_temperature = self._io.read_s2le()
            self.yrate_sensor_temperature = self._io.read_s2le()
            self.zrate_sensor_temperature = self._io.read_s2le()
            self.x_angular_rate = self._io.read_s2le()
            self.y_angular_rate = self._io.read_s2le()
            self.z_angular_rate = self._io.read_s2le()
            self.x_wheelspeed = self._io.read_s2le()
            self.y_wheelspeed = self._io.read_s2le()
            self.z_wheelspeed = self._io.read_s2le()


    class DhabisatHeaderT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.data_type = self._io.read_u1()
            self.time_stamp = self._io.read_u4le()
            self.file_index = self._io.read_u2le()
            self.interval = self._io.read_u2le()
            self.total_packets = self._io.read_u2le()
            self.current_packet = self._io.read_u2le()


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Dhabisat.Callsign(_io__raw_callsign_ror, self, self._root)


    @property
    def frame_length(self):
        if hasattr(self, '_m_frame_length'):
            return self._m_frame_length

        self._m_frame_length = self._io.size()
        return getattr(self, '_m_frame_length', None)





# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Sr0(KaitaiStruct):
    """:field callsign_lora: id1.id2.callsign_lora
    :field frame_number: id1.id2.frame_number
    :field message_type: id1.id2.message_type
    :field transmission_power: id1.id2.transmission_power
    :field satellite_unix_time: id1.id2.satellite_unix_time
    :field obc_temperature_celsius: id1.id2.obc_temperature_celsius
    :field battery_temperature_celsius: id1.id2.battery_temperature_celsius
    :field external_temperature_celsius: id1.id2.external_temperature_celsius
    :field base_plate_temperature_celsius: id1.id2.base_plate_temperature_celsius
    :field solar_panel_temperature_celsius: id1.id2.solar_panel_temperature_celsius
    :field radiation_microsv_per_h: id1.id2.radiation_microsv_per_h
    :field bus_voltage_v: id1.id2.bus_voltage_v
    :field bus_current_a: id1.id2.bus_current_a
    :field battery_maximum_capacity_ah: id1.id2.battery_maximum_capacity_ah
    :field battery_remaining_capacity_ah: id1.id2.battery_remaining_capacity_ah
    :field solar_bus_voltage_v: id1.id2.solar_bus_voltage_v
    :field solar_bus_current_a: id1.id2.solar_bus_current_a
    :field boot_counter: id1.id2.boot_counter
    :field checksum: id1.id2.checksum
    :field dest_callsign: id1.id2.ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: id1.id2.ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: id1.id2.ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: id1.id2.ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field ctl: id1.id2.ax25_frame.ax25_header.ctl
    :field pid: id1.id2.ax25_frame.ax25_header.pid
    :field callsign_fsk: id1.id2.ax25_frame.ax25_payload.callsign_fsk
    :field frame_number: id1.id2.ax25_frame.ax25_payload.frame_number
    :field message_type: id1.id2.ax25_frame.ax25_payload.message_type
    :field transmission_power: id1.id2.ax25_frame.ax25_payload.transmission_power
    :field satellite_unix_time: id1.id2.ax25_frame.ax25_payload.satellite_unix_time
    :field obc_temperature_celsius: id1.id2.ax25_frame.ax25_payload.obc_temperature_celsius
    :field battery_temperature_celsius: id1.id2.ax25_frame.ax25_payload.battery_temperature_celsius
    :field external_temperature_celsius: id1.id2.ax25_frame.ax25_payload.external_temperature_celsius
    :field base_plate_temperature_celsius: id1.id2.ax25_frame.ax25_payload.base_plate_temperature_celsius
    :field solar_panel_temperature_celsius: id1.id2.ax25_frame.ax25_payload.solar_panel_temperature_celsius
    :field radiation_microsv_per_h: id1.id2.ax25_frame.ax25_payload.radiation_microsv_per_h
    :field bus_voltage_v: id1.id2.ax25_frame.ax25_payload.bus_voltage_v
    :field bus_current_a: id1.id2.ax25_frame.ax25_payload.bus_current_a
    :field battery_maximum_capacity_ah: id1.id2.ax25_frame.ax25_payload.battery_maximum_capacity_ah
    :field battery_remaining_capacity_ah: id1.id2.ax25_frame.ax25_payload.battery_remaining_capacity_ah
    :field solar_bus_voltage_v: id1.id2.ax25_frame.ax25_payload.solar_bus_voltage_v
    :field solar_bus_current_a: id1.id2.ax25_frame.ax25_payload.solar_bus_current_a
    :field boot_counter: id1.id2.ax25_frame.ax25_payload.boot_counter
    :field checksum: id1.id2.ax25_frame.ax25_payload.checksum
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.id1 = Sr0.Type1(self._io, self, self._root)

    class Type1(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            _on = self.lora_or_fsk
            if _on == 1397895251:
                self.id2 = Sr0.Lora(self._io, self, self._root)
            elif _on == 2795545750:
                self.id2 = Sr0.Fsk(self._io, self, self._root)

        @property
        def lora_or_fsk(self):
            if hasattr(self, '_m_lora_or_fsk'):
                return self._m_lora_or_fsk

            _pos = self._io.pos()
            self._io.seek(0)
            self._m_lora_or_fsk = self._io.read_u4be()
            self._io.seek(_pos)
            return getattr(self, '_m_lora_or_fsk', None)


    class Lora(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign_lora = (self._io.read_bytes(6)).decode(u"ASCII")
            if not  ((self.callsign_lora == u"SR0SAT")) :
                raise kaitaistruct.ValidationNotAnyOfError(self.callsign_lora, self._io, u"/types/lora/seq/0")
            self.frame_number = self._io.read_u2le()
            self.message_type = self._io.read_u1()
            self.transmission_power = self._io.read_u1()
            self.satellite_unix_time = self._io.read_u4le()
            self.obc_temperature = self._io.read_s2le()
            self.battery_temperature = self._io.read_s2le()
            self.external_temperature = self._io.read_s2le()
            self.base_plate_temperature = self._io.read_s2le()
            self.solar_panel_temperature = self._io.read_s2le()
            self.radiation = self._io.read_s2le()
            self.bus_voltage = self._io.read_u2le()
            self.bus_current = self._io.read_s2le()
            self.battery_maximum_capacity = self._io.read_u2le()
            self.battery_remaining_capacity = self._io.read_u2le()
            self.solar_bus_voltage = self._io.read_u2le()
            self.solar_bus_current = self._io.read_u2le()
            self.boot_counter = self._io.read_u2le()
            self.checksum = self._io.read_u1()

        @property
        def battery_temperature_celsius(self):
            if hasattr(self, '_m_battery_temperature_celsius'):
                return self._m_battery_temperature_celsius

            self._m_battery_temperature_celsius = (self.battery_temperature / 10.0)
            return getattr(self, '_m_battery_temperature_celsius', None)

        @property
        def solar_bus_current_a(self):
            if hasattr(self, '_m_solar_bus_current_a'):
                return self._m_solar_bus_current_a

            self._m_solar_bus_current_a = (self.solar_bus_current / 1000.0)
            return getattr(self, '_m_solar_bus_current_a', None)

        @property
        def base_plate_temperature_celsius(self):
            if hasattr(self, '_m_base_plate_temperature_celsius'):
                return self._m_base_plate_temperature_celsius

            self._m_base_plate_temperature_celsius = (self.base_plate_temperature / 10.0)
            return getattr(self, '_m_base_plate_temperature_celsius', None)

        @property
        def solar_panel_temperature_celsius(self):
            if hasattr(self, '_m_solar_panel_temperature_celsius'):
                return self._m_solar_panel_temperature_celsius

            self._m_solar_panel_temperature_celsius = (self.solar_panel_temperature / 10.0)
            return getattr(self, '_m_solar_panel_temperature_celsius', None)

        @property
        def bus_current_a(self):
            if hasattr(self, '_m_bus_current_a'):
                return self._m_bus_current_a

            self._m_bus_current_a = (self.bus_current / 1000.0)
            return getattr(self, '_m_bus_current_a', None)

        @property
        def solar_bus_voltage_v(self):
            if hasattr(self, '_m_solar_bus_voltage_v'):
                return self._m_solar_bus_voltage_v

            self._m_solar_bus_voltage_v = (self.solar_bus_voltage / 1000.0)
            return getattr(self, '_m_solar_bus_voltage_v', None)

        @property
        def battery_remaining_capacity_ah(self):
            if hasattr(self, '_m_battery_remaining_capacity_ah'):
                return self._m_battery_remaining_capacity_ah

            self._m_battery_remaining_capacity_ah = (self.battery_remaining_capacity / 1000.0)
            return getattr(self, '_m_battery_remaining_capacity_ah', None)

        @property
        def obc_temperature_celsius(self):
            if hasattr(self, '_m_obc_temperature_celsius'):
                return self._m_obc_temperature_celsius

            self._m_obc_temperature_celsius = (self.obc_temperature / 10.0)
            return getattr(self, '_m_obc_temperature_celsius', None)

        @property
        def external_temperature_celsius(self):
            if hasattr(self, '_m_external_temperature_celsius'):
                return self._m_external_temperature_celsius

            self._m_external_temperature_celsius = (self.external_temperature / 10.0)
            return getattr(self, '_m_external_temperature_celsius', None)

        @property
        def bus_voltage_v(self):
            if hasattr(self, '_m_bus_voltage_v'):
                return self._m_bus_voltage_v

            self._m_bus_voltage_v = (self.bus_voltage / 1000.0)
            return getattr(self, '_m_bus_voltage_v', None)

        @property
        def battery_maximum_capacity_ah(self):
            if hasattr(self, '_m_battery_maximum_capacity_ah'):
                return self._m_battery_maximum_capacity_ah

            self._m_battery_maximum_capacity_ah = (self.battery_maximum_capacity / 1000.0)
            return getattr(self, '_m_battery_maximum_capacity_ah', None)

        @property
        def radiation_microsv_per_h(self):
            if hasattr(self, '_m_radiation_microsv_per_h'):
                return self._m_radiation_microsv_per_h

            self._m_radiation_microsv_per_h = (self.radiation / 100.0)
            return getattr(self, '_m_radiation_microsv_per_h', None)


    class Fsk(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_frame = Sr0.Fsk.Ax25Frame(self._io, self, self._root)

        class Ax25Frame(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.ax25_header = Sr0.Fsk.Ax25Header(self._io, self, self._root)
                self.ax25_payload = Sr0.Fsk.Ax25Payload(self._io, self, self._root)


        class Ax25Payload(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.callsign_fsk = (self._io.read_bytes(6)).decode(u"ASCII")
                if not  ((self.callsign_fsk == u"SR0SAT")) :
                    raise kaitaistruct.ValidationNotAnyOfError(self.callsign_fsk, self._io, u"/types/fsk/types/ax25_payload/seq/0")
                self.frame_number = self._io.read_u2le()
                self.message_type = self._io.read_u1()
                self.transmission_power = self._io.read_u1()
                self.satellite_unix_time = self._io.read_u4le()
                self.obc_temperature = self._io.read_s2le()
                self.battery_temperature = self._io.read_s2le()
                self.external_temperature = self._io.read_s2le()
                self.base_plate_temperature = self._io.read_s2le()
                self.solar_panel_temperature = self._io.read_s2le()
                self.radiation = self._io.read_s2le()
                self.bus_voltage = self._io.read_u2le()
                self.bus_current = self._io.read_s2le()
                self.battery_maximum_capacity = self._io.read_u2le()
                self.battery_remaining_capacity = self._io.read_u2le()
                self.solar_bus_voltage = self._io.read_u2le()
                self.solar_bus_current = self._io.read_u2le()
                self.boot_counter = self._io.read_u2le()
                self.checksum = self._io.read_u1()

            @property
            def battery_temperature_celsius(self):
                if hasattr(self, '_m_battery_temperature_celsius'):
                    return self._m_battery_temperature_celsius

                self._m_battery_temperature_celsius = (self.battery_temperature / 10.0)
                return getattr(self, '_m_battery_temperature_celsius', None)

            @property
            def solar_bus_current_a(self):
                if hasattr(self, '_m_solar_bus_current_a'):
                    return self._m_solar_bus_current_a

                self._m_solar_bus_current_a = (self.solar_bus_current / 1000.0)
                return getattr(self, '_m_solar_bus_current_a', None)

            @property
            def base_plate_temperature_celsius(self):
                if hasattr(self, '_m_base_plate_temperature_celsius'):
                    return self._m_base_plate_temperature_celsius

                self._m_base_plate_temperature_celsius = (self.base_plate_temperature / 10.0)
                return getattr(self, '_m_base_plate_temperature_celsius', None)

            @property
            def solar_panel_temperature_celsius(self):
                if hasattr(self, '_m_solar_panel_temperature_celsius'):
                    return self._m_solar_panel_temperature_celsius

                self._m_solar_panel_temperature_celsius = (self.solar_panel_temperature / 10.0)
                return getattr(self, '_m_solar_panel_temperature_celsius', None)

            @property
            def bus_current_a(self):
                if hasattr(self, '_m_bus_current_a'):
                    return self._m_bus_current_a

                self._m_bus_current_a = (self.bus_current / 1000.0)
                return getattr(self, '_m_bus_current_a', None)

            @property
            def solar_bus_voltage_v(self):
                if hasattr(self, '_m_solar_bus_voltage_v'):
                    return self._m_solar_bus_voltage_v

                self._m_solar_bus_voltage_v = (self.solar_bus_voltage / 1000.0)
                return getattr(self, '_m_solar_bus_voltage_v', None)

            @property
            def battery_remaining_capacity_ah(self):
                if hasattr(self, '_m_battery_remaining_capacity_ah'):
                    return self._m_battery_remaining_capacity_ah

                self._m_battery_remaining_capacity_ah = (self.battery_remaining_capacity / 1000.0)
                return getattr(self, '_m_battery_remaining_capacity_ah', None)

            @property
            def obc_temperature_celsius(self):
                if hasattr(self, '_m_obc_temperature_celsius'):
                    return self._m_obc_temperature_celsius

                self._m_obc_temperature_celsius = (self.obc_temperature / 10.0)
                return getattr(self, '_m_obc_temperature_celsius', None)

            @property
            def external_temperature_celsius(self):
                if hasattr(self, '_m_external_temperature_celsius'):
                    return self._m_external_temperature_celsius

                self._m_external_temperature_celsius = (self.external_temperature / 10.0)
                return getattr(self, '_m_external_temperature_celsius', None)

            @property
            def bus_voltage_v(self):
                if hasattr(self, '_m_bus_voltage_v'):
                    return self._m_bus_voltage_v

                self._m_bus_voltage_v = (self.bus_voltage / 1000.0)
                return getattr(self, '_m_bus_voltage_v', None)

            @property
            def battery_maximum_capacity_ah(self):
                if hasattr(self, '_m_battery_maximum_capacity_ah'):
                    return self._m_battery_maximum_capacity_ah

                self._m_battery_maximum_capacity_ah = (self.battery_maximum_capacity / 1000.0)
                return getattr(self, '_m_battery_maximum_capacity_ah', None)

            @property
            def radiation_microsv_per_h(self):
                if hasattr(self, '_m_radiation_microsv_per_h'):
                    return self._m_radiation_microsv_per_h

                self._m_radiation_microsv_per_h = (self.radiation / 100.0)
                return getattr(self, '_m_radiation_microsv_per_h', None)


        class Ax25Header(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.dest_callsign_raw = Sr0.Fsk.CallsignRaw(self._io, self, self._root)
                self.dest_ssid_raw = Sr0.Fsk.SsidMask(self._io, self, self._root)
                self.src_callsign_raw = Sr0.Fsk.CallsignRaw(self._io, self, self._root)
                self.src_ssid_raw = Sr0.Fsk.SsidMask(self._io, self, self._root)
                self.ctl = self._io.read_u1()
                self.pid = self._io.read_u1()


        class Callsign(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


        class SsidMask(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.ssid_mask = self._io.read_u1()

            @property
            def ssid(self):
                if hasattr(self, '_m_ssid'):
                    return self._m_ssid

                self._m_ssid = ((self.ssid_mask & 15) >> 1)
                return getattr(self, '_m_ssid', None)


        class CallsignRaw(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self._raw__raw_callsign_ror = self._io.read_bytes(6)
                self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
                _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
                self.callsign_ror = Sr0.Fsk.Callsign(_io__raw_callsign_ror, self, self._root)




